<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Service\Sm00005000ParametrizacoesServicesService;
use Teknisa\SM\Util\Utilities;

class Sm00005000ParametrizacoesServicesController extends SMCrud {
    
    protected $dataSourceName = 'fct_parametrogeral_widget';
    protected $serviceParamSm;
    
    public function __construct($crudService, $environment, Sm00005000ParametrizacoesServicesService $serviceParamSm) {
        parent::__construct($crudService, $environment);
        $this->serviceParamSm  = $serviceParamSm;
    }
    
    /**
     * A função "buscaNovoCodigo" busca os códigos para validação
     */

    public function find(Request\Filter $request, Response $response) {
        try {
            
            parent::find($request, $response);

            $rows = $response->getDataSets()[0]->getRows();
            $rows[0]["FCT_ALTCONTRAPROVPERF"] = Utilities::getArrayDeString($rows[0]["FCT_ALTCONTRAPROVPERF"], true);
            $rows[0]["FCT_ALTPROPCONCPERF"] = Utilities::getArrayDeString($rows[0]["FCT_ALTPROPCONCPERF"], true);

            $response->setDataSets(array(new DataSet($this->dataSourceName, $rows)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function buscaNovoCodigo(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $cdContador = $row['CDCONTADOR'];
    
            $buscaNovoCod = $this->serviceParamSm->buscaNovoCodigo($cdContador);
            
            $dtsNovoCod = new DataSet("SEQUENCIAL", $buscaNovoCod);
            
            $response->addDataSet($dtsNovoCod);

        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    /**
     * A função "atualizaNovoCodigo" atualiza o novo codigo do contrato ou da proposta
     */
    
    public function atualizaNovoCodigo(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $cdContador = $row['CDCONTADOR'];
            $nrSequencial = $row['NRSEQUENCIAL'];
    
            $this->serviceParamSm->atualizaNovoCodigo($cdContador, $nrSequencial);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    /**
     * A função "insereNovoCodigo" insere o novo codigo do contrato ou da proposta
     */
    
    public function insereNovoCodigo(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $cdContador = $row['CDCONTADOR'];
            $nrSequencial = $row['NRSEQUENCIAL'];
    
            $this->serviceParamSm->insereNovoCodigo($cdContador, $nrSequencial);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveperfilacesso (Request $request, Response $response) {
        $row = $request->getRow();
        $perfilacessoProposta = empty($row["FCT_ALTPROPCONCPERF"]) ? null : $row["FCT_ALTPROPCONCPERF"];
        $perfilacessoContrato = empty($row["FCT_ALTCONTRAPROVPERF"]) ? null : $row["FCT_ALTCONTRAPROVPERF"];
        $nrorg = $row["NRORG"];
        $nrorginclusao = $row["NRORGINCLUSAO"];
        $cdoperinclusao = $row["CDOPERINCLUSAO"];
        $cdoperultatu = $row["CDOPERULTATU"];
        $nrorgultatu = $row["NRORGULTATU"];
        $dtinclusao = $row["DTINCLUSAO"];
        $dtultatu = $row["DTULTATU"];
        $idativo = $row["IDATIVO"];
        $proposta = null;
        $contrato = null;

        try {
            $buscaEmPerfisContratoExistentes = $this->serviceParamSm->buscarperfilacesso($nrorg, 'C');
            $buscaEmPerfisPropostaExistentes = $this->serviceParamSm->buscarperfilacesso($nrorg, 'P');

            $perfisContratoExistentes = array();
            $perfisPropostaExistentes = array();

            foreach($buscaEmPerfisContratoExistentes as $removedorarray){array_push($perfisContratoExistentes,$removedorarray["NRPERFILACESSO"]);}
            foreach($buscaEmPerfisPropostaExistentes as $removedorarray){array_push($perfisPropostaExistentes,$removedorarray["NRPERFILACESSO"]);}
            if($perfilacessoProposta == NULL){
                $perfilacessoProposta = array();
            }
            if($perfilacessoContrato == NULL){
                $perfilacessoContrato = array();
            }
            // PROPOSTA se existe algum perfil na row que não existe no banco
            foreach($perfilacessoProposta as $perfil) {
                if (!in_array($perfil, $perfisPropostaExistentes)){
                    $proposta = true;
                    $contrato = false;
                    $idperfilacesso = $this->serviceParamSm->saveperfilacesso($perfil, $nrorg, $nrorginclusao, $cdoperinclusao, $cdoperultatu, $nrorgultatu, $dtinclusao, $dtultatu, $idativo, $proposta, $contrato);
                }
            }

            //PROPOSTA se existe algum perfil no banco que ofi removido da row
            foreach($perfisPropostaExistentes as $perfil) {
                if (!in_array($perfil, $perfilacessoProposta)){
                    $proposta = true;
                    $contrato = false;
                    $idperfilacesso = $this->serviceParamSm->deleteperfilacesso($perfil, $proposta, $contrato);
                }
            }

            //CONTRATO se existe algum perfil na row que não existe no banco
            foreach($perfilacessoContrato as $perfil) {
                if (!in_array($perfil, $perfisContratoExistentes)){
                    $contrato = true;
                    $proposta = false;
                    $idperfilacesso = $this->serviceParamSm->saveperfilacesso($perfil, $nrorg, $nrorginclusao, $cdoperinclusao, $cdoperultatu, $nrorgultatu, $dtinclusao, $dtultatu, $idativo, $proposta, $contrato);
                }
            }

            //CONTRATO se existe algum perfil no banco que ofi removido da row
            foreach($perfisContratoExistentes as $perfil) {
                if (!in_array($perfil, $perfilacessoContrato)){
                    $contrato = true;
                    $proposta = false;
                    $idperfilacesso = $this->serviceParamSm->deleteperfilacesso($perfil, $proposta, $contrato);
                }
            }
        }catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function validarperfilacesso (Request $request, Response $response){
        try {
            $result = false;
            $row                 = $request->getRow();
            $nrparcnegocio       = $row[0];
            $nrorg               = $row[1];
            $derivadode          = $row[2];
            $perfisacessousuario = [];

            $perfilacessousuario = $this->serviceParamSm->validarperfilacesso($nrparcnegocio);
            $perfilacessoparametrizado = $this->serviceParamSm->buscarperfilacesso($nrorg, $derivadode);
            
            foreach($perfilacessousuario as $perfilU){
                array_push($perfisacessousuario, $perfilU["NRPERFILACESSO"]);
            }
            
            if($perfilacessoparametrizado){
                $perfilacessoparametrizadoarray = array();
                foreach($perfilacessoparametrizado as $perfil){
                    array_push($perfilacessoparametrizadoarray,$perfil["NRPERFILACESSO"]);
                }
                
                foreach($perfisacessousuario as $perfilUs) {
                    if (in_array($perfilUs, $perfilacessoparametrizadoarray)) {
                        $result = true;
                    }
                }
            }else{
                $result = true;
            }
            $response->setDataSets(array(new DataSet($this->dataSourceName, [$result])));
        }catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
     /**
     * A função "completaPercentual" verifica qual o percentual faltante para inserção na página "índices Financeiros Padrões"
     */
    
    public function completaPercentual(Request\Row $request, Response $response) {
        try{
    
            $result = $this->serviceParamSm->completaPercentual();
            
            $response->addDataSet(new DataSet("completaPercentual", array("PERCENTUAL" => $result)));

        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     /**
     * A função "validaPercentual" valida o percentual para inserção na página "índices Financeiros Padrões"
     */
    
    public function validaPercentual(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            $nrpercentindice = $row['NRPERCENTINDICE'];
            $cdindice        = $row['CDINDICE'];
            
            $result = $this->serviceParamSm->validaPercentual($nrpercentindice, $cdindice);
            $response->addDataSet(new DataSet("validaPercentual", array("PERCENTUAL" => $result)));

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     /**
     * A função "cadAutoSignatPadr" realiza o cadastramento automático dos signatários padrões da parametrização SM
     */
    
    public function cadAutoSignatPadr(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $result = $this->serviceParamSm->cadAutoSignatPadr($row);
            $response->addDataSet(new DataSet("cadauto", array("retorno" => $result)));

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * A função "validaSignatPadr" realiza a validação dos signatários padrões para evitar a inclusão repetida de um parceiro.
     */
    
    public function validaSignatPadr(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $result = $this->serviceParamSm->validaSignatPadr($row);
            $response->addDataSet(new DataSet("validaSignatPadr", array("valida" => $result)));

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * A função "insereOperadoresNotifica" realiza a inserção dos operadores do sistema que devem ser notificados.
     */
    public function insereOperadoresNotifica(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $nrsignatariospadr = $row["NRSIGNATARIOSPADR"];
            $operadores = $row["OPERADORES"];
            
            $this->serviceParamSm->insereOperadoresNotifica($nrsignatariospadr, $operadores);

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}
?>